@@ -24,8 +24,11 @@
 
 #include <govirt/ovirt-action-rest-call.h>
 #include <govirt/ovirt-api-private.h>
+#include <govirt/ovirt-cluster-private.h>
 #include <govirt/ovirt-collection-private.h>
+#include <govirt/ovirt-data-center-private.h>
 #include <govirt/ovirt-enum-types-private.h>
+#include <govirt/ovirt-host-private.h>
 #include <govirt/ovirt-proxy-private.h>
 #include <govirt/ovirt-resource-private.h>
 #include <govirt/ovirt-resource-rest-call.h>
@@ -33,6 +36,5 @@
 #include <govirt/ovirt-storage-domain-private.h>
 #include <govirt/ovirt-utils.h>
 #include <govirt/ovirt-vm-private.h>
-#include <govirt/glib-compat.h>
 
 #endif /* __OVIRT_PRIVATE_H__ */
