@@ -28,15 +28,13 @@
 #include "ovirt-proxy-private.h"
 #include "ovirt-resource-private.h"
 #include "ovirt-resource-rest-call.h"
-
-#define OVIRT_CDROM_GET_PRIVATE(obj)                         \
-        (G_TYPE_INSTANCE_GET_PRIVATE((obj), OVIRT_TYPE_CDROM, OvirtCdromPrivate))
+#include "ovirt-utils.h"
 
 struct _OvirtCdromPrivate {
     char *file;
 };
 
-G_DEFINE_TYPE(OvirtCdrom, ovirt_cdrom, OVIRT_TYPE_RESOURCE);
+G_DEFINE_TYPE_WITH_PRIVATE(OvirtCdrom, ovirt_cdrom, OVIRT_TYPE_RESOURCE);
 
 
 enum {
@@ -95,25 +93,28 @@ static void ovirt_cdrom_finalize(GObject *object)
 }
 
 
-static gboolean ovirt_cdrom_refresh_from_xml(OvirtCdrom *cdrom,
-                                             RestXmlNode *node)
+static gboolean ovirt_cdrom_init_from_xml(OvirtResource *resource,
+                                          RestXmlNode *node,
+                                          GError **error)
 {
-    RestXmlNode *file_node;
-    const char *file;
-    const char *file_key = g_intern_string("file");
     char *name;
+    OvirtResourceClass *parent_class;
+    OvirtXmlElement cdrom_elements[] = {
+        { .prop_name = "file",
+          .xml_path = "file",
+          .xml_attr = "id",
+        },
+        { NULL , },
+    };
 
-    file_node = g_hash_table_lookup(node->children, file_key);
-    if (file_node != NULL) {
-        file = rest_xml_node_get_attr(file_node, "id");
-        if (g_strcmp0(file, cdrom->priv->file) != 0) {
-            g_free(cdrom->priv->file);
-            cdrom->priv->file = g_strdup(file);
-            g_object_notify(G_OBJECT(cdrom), "file");
-        }
-    }
+    parent_class = OVIRT_RESOURCE_CLASS(ovirt_cdrom_parent_class);
 
-    g_object_get(G_OBJECT(cdrom), "name", &name, NULL);
+    if (!parent_class->init_from_xml(resource, node, error))
+        return FALSE;
+
+    ovirt_rest_xml_node_parse(node, G_OBJECT(resource), cdrom_elements);
+
+    g_object_get(G_OBJECT(resource), "name", &name, NULL);
     if (name == NULL) {
         /* Build up fake name as ovirt_collection_refresh_from_xml()
          * expects it to be set (it uses it as a hash table key), but
@@ -122,32 +123,14 @@ static gboolean ovirt_cdrom_refresh_from_xml(OvirtCdrom *cdrom,
          * enough for now
          */
         g_debug("Setting fake 'name' for cdrom resource");
-        g_object_set(G_OBJECT(cdrom), "name", "cdrom0", NULL);
-    } else {
-        g_free(name);
+        g_object_set(G_OBJECT(resource), "name", "cdrom0", NULL);
     }
 
+    g_free(name);
     return TRUE;
 }
 
 
-static gboolean ovirt_cdrom_init_from_xml(OvirtResource *resource,
-                                          RestXmlNode *node,
-                                          GError **error)
-{
-    gboolean parsed_ok;
-    OvirtResourceClass *parent_class;
-
-    parsed_ok = ovirt_cdrom_refresh_from_xml(OVIRT_CDROM(resource), node);
-    if (!parsed_ok) {
-        return FALSE;
-    }
-    parent_class = OVIRT_RESOURCE_CLASS(ovirt_cdrom_parent_class);
-
-    return parent_class->init_from_xml(resource, node, error);
-}
-
-
 static char *ovirt_cdrom_to_xml(OvirtResource *resource)
 {
     OvirtCdrom *cdrom;
@@ -170,8 +153,6 @@ static void ovirt_cdrom_class_init(OvirtCdromClass *klass)
     OvirtResourceClass *resource_class = OVIRT_RESOURCE_CLASS(klass);
     GParamSpec *param_spec;
 
-    g_type_class_add_private(klass, sizeof(OvirtCdromPrivate));
-
     resource_class->init_from_xml = ovirt_cdrom_init_from_xml;
     resource_class->to_xml = ovirt_cdrom_to_xml;
     resource_class->add_rest_params = ovirt_cdrom_add_rest_params;
@@ -193,7 +174,7 @@ static void ovirt_cdrom_class_init(OvirtCdromClass *klass)
 
 static void ovirt_cdrom_init(OvirtCdrom *cdrom)
 {
-    cdrom->priv = OVIRT_CDROM_GET_PRIVATE(cdrom);
+    cdrom->priv = ovirt_cdrom_get_instance_private(cdrom);
 }
 
 
