@@ -30,9 +30,6 @@
 #include "ovirt-error.h"
 #include "govirt-private.h"
 
-#define OVIRT_COLLECTION_GET_PRIVATE(obj)                         \
-        (G_TYPE_INSTANCE_GET_PRIVATE((obj), OVIRT_TYPE_COLLECTION, OvirtCollectionPrivate))
-
 struct _OvirtCollectionPrivate {
     char *href;
     char *collection_xml_name;
@@ -42,7 +39,7 @@ struct _OvirtCollectionPrivate {
     GHashTable *resources;
 };
 
-G_DEFINE_TYPE(OvirtCollection, ovirt_collection, G_TYPE_OBJECT);
+G_DEFINE_TYPE_WITH_PRIVATE(OvirtCollection, ovirt_collection, G_TYPE_OBJECT);
 
 
 enum {
@@ -129,8 +126,6 @@ static void ovirt_collection_class_init(OvirtCollectionClass *klass)
     GObjectClass *object_class = G_OBJECT_CLASS(klass);
     GParamSpec *param_spec;
 
-    g_type_class_add_private(klass, sizeof(OvirtCollectionPrivate));
-
     object_class->finalize = ovirt_collection_finalize;
     object_class->get_property = ovirt_collection_get_property;
     object_class->set_property = ovirt_collection_set_property;
@@ -193,7 +188,7 @@ static void ovirt_collection_class_init(OvirtCollectionClass *klass)
 
 static void ovirt_collection_init(OvirtCollection *collection)
 {
-    collection->priv = OVIRT_COLLECTION_GET_PRIVATE(collection);
+    collection->priv = ovirt_collection_get_instance_private(collection);
 }
 
 /**
@@ -235,9 +230,7 @@ ovirt_collection_new_resource_from_xml(OvirtCollection *collection,
                                        RestXmlNode *node,
                                        GError **error)
 {
-    return OVIRT_RESOURCE(g_initable_new(collection->priv->resource_type,
-                                         NULL, error,
-                                         "xml-node", node , NULL));
+    return ovirt_resource_new_from_xml(collection->priv->resource_type, node, error);
 }
 
 
@@ -344,6 +337,52 @@ OvirtCollection *ovirt_collection_new_from_xml(RestXmlNode *root_node,
 }
 
 
+OvirtCollection *ovirt_sub_collection_new_from_resource(OvirtResource *resource,
+                                                        const char *href,
+                                                        const char *collection_name,
+                                                        GType resource_type,
+                                                        const char *resource_name)
+{
+    const char *link = ovirt_resource_get_sub_collection(resource, href);
+
+    if (link == NULL)
+        return NULL;
+
+    return ovirt_collection_new(link, collection_name, resource_type, resource_name);
+}
+
+OvirtCollection *ovirt_sub_collection_new_from_resource_search(OvirtResource *resource,
+                                                               const char *href,
+                                                               const char *collection_name,
+                                                               GType resource_type,
+                                                               const char *resource_name,
+                                                               const char *query)
+{
+    const char *link;
+    char *substr;
+    gchar *link_query, *escaped_query;
+    OvirtCollection *collection;
+
+    link = ovirt_resource_get_sub_collection(resource, href);
+    if (link == NULL)
+        return NULL;
+
+    /* link is will be something like "/ovirt-engine/api/vms?search={query}", so
+     * we need to strip out {query} substring.
+     */
+    substr = g_strrstr(link, "{query}");
+    if (substr != NULL)
+        *substr = '\0';
+
+    escaped_query = g_uri_escape_string(query, NULL, FALSE);
+    link_query = g_strconcat(link, escaped_query, NULL);
+    collection = ovirt_collection_new(link_query, collection_name, resource_type, resource_name);
+    g_free(escaped_query);
+    g_free(link_query);
+
+    return collection;
+}
+
 /**
  * ovirt_collection_fetch:
  * @collection: a #OvirtCollection
