@@ -40,9 +40,7 @@
 #include <rest/rest-xml-node.h>
 #include <rest/rest-xml-parser.h>
 
-G_DEFINE_TYPE (OvirtProxy, ovirt_proxy, REST_TYPE_PROXY);
-
-#define OVIRT_PROXY_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE((o), OVIRT_TYPE_PROXY, OvirtProxyPrivate))
+G_DEFINE_TYPE_WITH_PRIVATE (OvirtProxy, ovirt_proxy, REST_TYPE_PROXY);
 
 enum {
     PROP_0,
@@ -155,8 +153,6 @@ static RestProxyCall *ovirt_rest_call_new(OvirtProxy *proxy,
         rest_proxy_call_set_method(call, method);
     }
     rest_proxy_call_set_function(call, href);
-    /* FIXME: to set or not to set ?? */
-    rest_proxy_call_add_header(call, "All-Content", "true");
 
     return call;
 }
@@ -218,8 +214,17 @@ static void ovirt_proxy_call_async_data_free(OvirtProxyCallAsyncData *data)
             g_object_unref(G_OBJECT(data->result));
         }
         if ((data->cancellable != NULL) && (data->cancellable_cb_id != 0)) {
-            g_cancellable_disconnect(data->cancellable, data->cancellable_cb_id);
+            if (g_cancellable_is_cancelled(data->cancellable)) {
+                /* Cancellable has already been cancelled, we don't need to use
+                 * g_cancellable_disconnect() to disconnect the signal handler
+                 * as we know the 'cancelled' signal is no longer going to be emitted
+                 */
+                g_signal_handler_disconnect(data->cancellable, data->cancellable_cb_id);
+            } else {
+                g_cancellable_disconnect(data->cancellable, data->cancellable_cb_id);
+            }
         }
+        g_clear_object(&data->cancellable);
         g_slice_free(OvirtProxyCallAsyncData, data);
 }
 
@@ -231,6 +236,24 @@ call_async_cancelled_cb (G_GNUC_UNUSED GCancellable *cancellable,
 }
 
 
+static void rest_call_async_set_error(RestProxyCall *call, GSimpleAsyncResult *result, const GError *error)
+{
+    GError *local_error = NULL;
+    RestXmlNode *root = ovirt_rest_xml_node_from_call(call);
+
+    if (root != NULL && ovirt_utils_gerror_from_xml_fault(root, &local_error)) {
+        g_debug("ovirt_rest_call_async(): %s", local_error->message);
+        g_simple_async_result_set_from_error(result, local_error);
+        g_clear_error(&local_error);
+    } else {
+        g_simple_async_result_set_from_error(result, error);
+    }
+
+    if (root != NULL) {
+        rest_xml_node_unref(root);
+    }
+}
+
 static void
 call_async_cb(RestProxyCall *call, const GError *error,
               G_GNUC_UNUSED GObject *weak_object,
@@ -240,7 +263,7 @@ call_async_cb(RestProxyCall *call, const GError *error,
     GSimpleAsyncResult *result = data->result;
 
     if (error != NULL) {
-        g_simple_async_result_set_from_error(result, error);
+        rest_call_async_set_error(call, result, error);
     } else {
         GError *call_error = NULL;
         gboolean callback_result = TRUE;
@@ -250,7 +273,7 @@ call_async_cb(RestProxyCall *call, const GError *error,
                                                   data->call_user_data,
                                                   &call_error);
             if (call_error != NULL) {
-                g_simple_async_result_set_from_error(result, call_error);
+                rest_call_async_set_error(call, result, call_error);
             }
         }
 
@@ -284,7 +307,7 @@ void ovirt_rest_call_async(OvirtRestCall *call,
     data->call_user_data = user_data;
     data->destroy_call_data = destroy_func;
     if (cancellable != NULL) {
-        data->cancellable = cancellable;
+        data->cancellable = g_object_ref(cancellable);
         data->cancellable_cb_id = g_cancellable_connect(cancellable,
                                                         G_CALLBACK (call_async_cancelled_cb),
                                                         call, NULL);
@@ -338,6 +361,11 @@ static gboolean get_collection_xml_async_cb(OvirtProxy* proxy,
     data = (OvirtProxyGetCollectionAsyncData *)user_data;
 
     root = ovirt_rest_xml_node_from_call(call);
+    if (root == NULL) {
+        g_set_error_literal(error, OVIRT_ERROR, OVIRT_ERROR_PARSING_FAILED,
+                            _("Failed to parse response from collection"));
+        goto end;
+    }
 
     /* Do the parsing */
     g_warn_if_fail(data->parser != NULL);
@@ -347,6 +375,7 @@ static gboolean get_collection_xml_async_cb(OvirtProxy* proxy,
 
     rest_xml_node_unref(root);
 
+end:
     return parsed;
 }
 
@@ -458,7 +487,6 @@ static void ovirt_proxy_set_tmp_ca_file(OvirtProxy *proxy, const char *ca_file)
 }
 
 
-#if GLIB_CHECK_VERSION(2, 32, 0)
 static char *write_to_tmp_file(const char *template,
                                const char *data,
                                gsize data_len,
@@ -494,57 +522,6 @@ end:
 
     return result;
 }
-#else
-static char *write_to_tmp_file(const char *template,
-                               const char *data,
-                               gsize data_len,
-                               GError **error)
-{
-    int fd = -1;
-    char *tmp_file = NULL;
-    char *result = NULL;
-
-    fd = g_file_open_tmp(template, &tmp_file, error);
-    if (fd == -1) {
-        goto end;
-    }
-
-    while (data_len != 0) {
-        ssize_t bytes_written;
-        bytes_written = write(fd, data, data_len);
-        if (bytes_written == -1) {
-            if ((errno != EINTR) || (errno != EAGAIN)) {
-                g_set_error(error, G_FILE_ERROR,
-                            g_file_error_from_errno(errno),
-                            _("Failed to write to '%s': %s"),
-                            tmp_file, strerror(errno));
-                goto end;
-            }
-        }
-        g_assert(bytes_written <= (ssize_t)data_len);
-        data_len -= bytes_written;
-    }
-
-    result = tmp_file;
-    tmp_file = NULL;
-
-end:
-    if (fd != -1) {
-        int close_status = close(fd);
-        if (close_status != 0) {
-            g_set_error(error, G_FILE_ERROR,
-                        g_file_error_from_errno(errno),
-                        _("Failed to close '%s': %s"),
-                        result, strerror(errno));
-            g_free(result);
-            result = NULL;
-        }
-    }
-    g_free(tmp_file);
-    return result;
-}
-#endif
-
 
 static gboolean set_ca_cert_from_data(OvirtProxy *proxy,
                                       char *ca_cert_data,
@@ -801,9 +778,6 @@ static void ovirt_proxy_set_session_id(OvirtProxy *proxy, const char *session_id
         cookie = soup_cookie_new("JSESSIONID", session_id, domain, "/ovirt-engine/api", -1);
         soup_cookie_jar_add_cookie(proxy->priv->cookie_jar, cookie);
         proxy->priv->jsessionid_cookie = cookie;
-        ovirt_proxy_add_header(proxy, "Prefer", "persistent-auth");
-    } else {
-        ovirt_proxy_add_header(proxy, "Prefer", NULL);
     }
     g_free(url);
 }
@@ -815,6 +789,7 @@ static void ovirt_proxy_set_sso_token(OvirtProxy *proxy, const char *sso_token)
     g_free(proxy->priv->sso_token);
     proxy->priv->sso_token = g_strdup(sso_token);
 
+    ovirt_proxy_add_header(proxy, "Prefer", NULL);
     header_value = g_strdup_printf("Bearer %s", sso_token);
     ovirt_proxy_add_header(proxy, "Authorization", header_value);
     g_free(header_value);
@@ -903,6 +878,10 @@ static void ovirt_proxy_constructed(GObject *gobject)
         g_warning("Disabling strict checking of SSL certificates");
         g_object_set(OVIRT_PROXY(gobject), "ssl-strict", FALSE, NULL);
     }
+    /* FIXME: to set or not to set ?? */
+    ovirt_proxy_add_header(OVIRT_PROXY(gobject), "All-Content", "true");
+    ovirt_proxy_add_header(OVIRT_PROXY(gobject), "Prefer", "persistent-auth");
+    ovirt_proxy_add_header(OVIRT_PROXY(gobject), "Version", "3");
 
     /* Chain up to the parent class */
     if (G_OBJECT_CLASS(ovirt_proxy_parent_class)->constructed)
@@ -989,8 +968,6 @@ ovirt_proxy_class_init(OvirtProxyClass *klass)
                                                         NULL,
                                                         G_PARAM_READWRITE |
                                                         G_PARAM_STATIC_STRINGS));
-
-    g_type_class_add_private(klass, sizeof(OvirtProxyPrivate));
 }
 
 static void ssl_ca_file_changed(GObject *gobject,
@@ -1010,7 +987,7 @@ ovirt_proxy_init(OvirtProxy *self)
 {
     gulong handler_id;
 
-    self->priv = OVIRT_PROXY_GET_PRIVATE(self);
+    self->priv = ovirt_proxy_get_instance_private(self);
 
     handler_id = g_signal_connect(G_OBJECT(self), "notify::ssl-ca-file",
                                   (GCallback)ssl_ca_file_changed, NULL);
