@@ -35,9 +35,6 @@
 #include "ovirt-resource.h"
 #undef GOVIRT_UNSTABLE_API_ABI
 
-#define OVIRT_RESOURCE_GET_PRIVATE(obj)                         \
-        (G_TYPE_INSTANCE_GET_PRIVATE((obj), OVIRT_TYPE_RESOURCE, OvirtResourcePrivate))
-
 struct _OvirtResourcePrivate {
     char *guid;
     char *href;
@@ -60,7 +57,8 @@ static gboolean ovirt_resource_init_from_xml(OvirtResource *resource,
 
 G_DEFINE_TYPE_WITH_CODE(OvirtResource, ovirt_resource, G_TYPE_OBJECT,
                         G_IMPLEMENT_INTERFACE(G_TYPE_INITABLE,
-                                              ovirt_resource_initable_iface_init));
+                                              ovirt_resource_initable_iface_init);
+                        G_ADD_PRIVATE(OvirtResource));
 
 
 enum {
@@ -97,6 +95,15 @@ static void ovirt_resource_get_property(GObject *object,
     }
 }
 
+static void ovirt_resource_set_xml_node(OvirtResource *resource,
+                                        RestXmlNode *node)
+{
+    g_clear_pointer(&resource->priv->xml, &rest_xml_node_unref);
+    if (node != NULL) {
+        resource->priv->xml = rest_xml_node_ref(node);
+    }
+}
+
 static void ovirt_resource_set_property(GObject *object,
                                         guint prop_id,
                                         const GValue *value,
@@ -121,13 +128,10 @@ static void ovirt_resource_set_property(GObject *object,
         g_free(resource->priv->description);
         resource->priv->description = g_value_dup_string(value);
         break;
-    case PROP_XML_NODE: {
-        if (resource->priv->xml != NULL) {
-            g_boxed_free(REST_TYPE_XML_NODE, resource->priv->xml);
-        }
-        resource->priv->xml = g_value_dup_boxed(value);
+    case PROP_XML_NODE:
+        ovirt_resource_set_xml_node(OVIRT_RESOURCE(object),
+                                    g_value_get_boxed(value));
         break;
-    }
     default:
         G_OBJECT_WARN_INVALID_PROPERTY_ID(object, prop_id, pspec);
     }
@@ -198,8 +202,6 @@ static void ovirt_resource_class_init(OvirtResourceClass *klass)
 {
     GObjectClass *object_class = G_OBJECT_CLASS(klass);
 
-    g_type_class_add_private(klass, sizeof(OvirtResourcePrivate));
-
     klass->init_from_xml = ovirt_resource_init_from_xml_real;
     object_class->dispose = ovirt_resource_dispose;
     object_class->finalize = ovirt_resource_finalize;
@@ -251,7 +253,7 @@ static void ovirt_resource_class_init(OvirtResourceClass *klass)
 
 static void ovirt_resource_init(OvirtResource *resource)
 {
-    resource->priv = OVIRT_RESOURCE_GET_PRIVATE(resource);
+    resource->priv = ovirt_resource_get_instance_private(resource);
     resource->priv->actions = g_hash_table_new_full(g_str_hash, g_str_equal,
                                                     g_free, g_free);
     resource->priv->sub_collections = g_hash_table_new_full(g_str_hash,
@@ -422,7 +424,7 @@ static gboolean ovirt_resource_init_from_xml_real(OvirtResource *resource,
      */
     is_api = OVIRT_IS_API(resource);
 
-    g_return_val_if_fail(resource->priv->xml != NULL, FALSE);
+    g_return_val_if_fail(node != NULL, FALSE);
 
     guid = rest_xml_node_get_attr(node, "id");
     if ((guid == NULL) && !is_api) {
@@ -438,6 +440,7 @@ static gboolean ovirt_resource_init_from_xml_real(OvirtResource *resource,
         return FALSE;
     }
 
+    ovirt_resource_set_xml_node(resource, node);
     g_object_set(G_OBJECT(resource), "guid", guid, "href", href, NULL);
 
     ovirt_resource_set_name_from_xml(resource, node);
@@ -485,21 +488,22 @@ G_GNUC_INTERNAL RestXmlNode *ovirt_resource_rest_call_sync(OvirtRestCall *call,
         GError *local_error = NULL;
 
         root = ovirt_rest_xml_node_from_call(REST_PROXY_CALL(call));
-        ovirt_utils_gerror_from_xml_fault(root, &local_error);
+        if (root != NULL) {
+            ovirt_utils_gerror_from_xml_fault(root, &local_error);
+            rest_xml_node_unref(root);
+        }
         if (local_error != NULL) {
             g_clear_error(error);
             g_warning("Error while updating resource");
             g_warning("message: %s", local_error->message);
             g_propagate_error(error, local_error);
         }
-        if (root != NULL) {
-            rest_xml_node_unref(root);
-        }
+        g_warn_if_fail(error == NULL || *error != NULL);
 
         return NULL;
     }
 
-    return root;
+    return ovirt_rest_xml_node_from_call(REST_PROXY_CALL(call));
 }
 
 
@@ -860,17 +864,24 @@ static gboolean ovirt_resource_refresh_async_cb(OvirtProxy *proxy,
 {
     OvirtResource *resource;
     RestXmlNode *root;
-    gboolean refreshed;
+    gboolean refreshed = FALSE;
 
     g_return_val_if_fail(REST_IS_PROXY_CALL(call), FALSE);
     g_return_val_if_fail(OVIRT_IS_RESOURCE(user_data), FALSE);
 
     root = ovirt_rest_xml_node_from_call(call);
+    if (root == NULL) {
+        g_set_error_literal(error, OVIRT_ERROR, OVIRT_ERROR_PARSING_FAILED,
+                            _("Failed to parse response from resource"));
+        goto end;
+    }
+
     resource = OVIRT_RESOURCE(user_data);
     refreshed = ovirt_resource_init_from_xml(resource, root, error);
 
     rest_xml_node_unref(root);
 
+end:
     return refreshed;
 }
 
@@ -1065,3 +1076,50 @@ gboolean ovirt_resource_delete_finish(OvirtResource *resource,
 
     return ovirt_rest_call_finish(result, err);
 }
+
+
+static OvirtResource *ovirt_resource_new_valist(GType type, GError **error, const char *prop_name, ...)
+{
+    gpointer resource;
+    va_list var_args;
+    GError *local_error = NULL;
+
+    g_return_val_if_fail(g_type_is_a(type, OVIRT_TYPE_RESOURCE), NULL);
+
+    va_start(var_args, prop_name);
+    resource = g_initable_new_valist(type, prop_name, var_args, NULL, &local_error);
+    va_end(var_args);
+
+    if (local_error != NULL) {
+        g_warning("Failed to create resource of type %s: %s", g_type_name(type), local_error->message);
+        g_propagate_error(error, local_error);
+    }
+
+    return OVIRT_RESOURCE(resource);
+}
+
+
+G_GNUC_INTERNAL
+OvirtResource *ovirt_resource_new(GType type)
+{
+    return ovirt_resource_new_valist(type, NULL, NULL);
+}
+
+
+G_GNUC_INTERNAL
+OvirtResource *ovirt_resource_new_from_id(GType type, const char *id, const char *href)
+{
+    g_return_val_if_fail(id != NULL, NULL);
+    g_return_val_if_fail(href != NULL, NULL);
+
+    return ovirt_resource_new_valist(type, NULL, "guid", id, "href", href, NULL);
+}
+
+
+G_GNUC_INTERNAL
+OvirtResource *ovirt_resource_new_from_xml(GType type, RestXmlNode *node, GError **error)
+{
+    g_return_val_if_fail(node != NULL, NULL);
+
+    return ovirt_resource_new_valist(type, error, "xml-node", node, NULL);
+}
