@@ -25,11 +25,9 @@
 #include "ovirt-storage-domain.h"
 #include "govirt-private.h"
 
-#define OVIRT_STORAGE_DOMAIN_GET_PRIVATE(obj)                         \
-        (G_TYPE_INSTANCE_GET_PRIVATE((obj), OVIRT_TYPE_STORAGE_DOMAIN, OvirtStorageDomainPrivate))
-
 struct _OvirtStorageDomainPrivate {
     OvirtCollection *files;
+    GStrv data_center_ids;
 
     OvirtStorageDomainType type;
     gboolean is_master;
@@ -40,7 +38,7 @@ struct _OvirtStorageDomainPrivate {
     OvirtStorageDomainState state;
 };
 
-G_DEFINE_TYPE(OvirtStorageDomain, ovirt_storage_domain, OVIRT_TYPE_RESOURCE);
+G_DEFINE_TYPE_WITH_PRIVATE(OvirtStorageDomain, ovirt_storage_domain, OVIRT_TYPE_RESOURCE);
 
 enum {
     PROP_0,
@@ -50,13 +48,10 @@ enum {
     PROP_USED,
     PROP_COMMITTED,
     PROP_VERSION,
-    PROP_STATE
+    PROP_STATE,
+    PROP_DATA_CENTER_IDS,
 };
 
-static gboolean
-ovirt_storage_domain_refresh_from_xml(OvirtStorageDomain *domain,
-                                      RestXmlNode *node);
-
 static void ovirt_storage_domain_get_property(GObject *object,
                                               guint prop_id,
                                               GValue *value,
@@ -86,6 +81,9 @@ static void ovirt_storage_domain_get_property(GObject *object,
     case PROP_STATE:
         g_value_set_enum(value, domain->priv->state);
         break;
+    case PROP_DATA_CENTER_IDS:
+        g_value_set_boxed(value, domain->priv->data_center_ids);
+        break;
     default:
         G_OBJECT_WARN_INVALID_PROPERTY_ID(object, prop_id, pspec);
         break;
@@ -121,6 +119,10 @@ static void ovirt_storage_domain_set_property(GObject *object,
     case PROP_STATE:
         domain->priv->state = g_value_get_enum(value);
         break;
+     case PROP_DATA_CENTER_IDS:
+        g_strfreev(domain->priv->data_center_ids);
+        domain->priv->data_center_ids = g_value_dup_boxed(value);
+        break;
     default:
         G_OBJECT_WARN_INVALID_PROPERTY_ID(object, prop_id, pspec);
         break;
@@ -134,6 +136,7 @@ ovirt_storage_domain_dispose(GObject *obj)
     OvirtStorageDomain *domain = OVIRT_STORAGE_DOMAIN(obj);
 
     g_clear_object(&domain->priv->files);
+    g_clear_pointer(&domain->priv->data_center_ids, g_strfreev);
 
     G_OBJECT_CLASS(ovirt_storage_domain_parent_class)->dispose(obj);
 }
@@ -145,10 +148,36 @@ static gboolean ovirt_storage_domain_init_from_xml(OvirtResource *resource,
 {
     gboolean parsed_ok;
     OvirtResourceClass *parent_class;
-    OvirtStorageDomain *domain;
+    OvirtXmlElement storage_domain_elements[] = {
+        { .prop_name = "type",
+          .xml_path = "type",
+        },
+        { .prop_name = "master",
+          .xml_path = "master",
+        },
+        { .prop_name = "available",
+          .xml_path = "available",
+        },
+        { .prop_name = "used",
+          .xml_path = "used",
+        },
+        { .prop_name = "committed",
+          .xml_path = "committed",
+        },
+        { .prop_name = "version",
+          .xml_path = "storage_format",
+        },
+        { .prop_name = "state",
+          .xml_path = "status/state",
+        },
+        { .prop_name = "data-center-ids",
+          .xml_path = "data_centers",
+          .xml_attr = "id",
+        },
+        { NULL , }
+    };
 
-    domain = OVIRT_STORAGE_DOMAIN(resource);
-    parsed_ok = ovirt_storage_domain_refresh_from_xml(domain, node);
+    parsed_ok = ovirt_rest_xml_node_parse(node, G_OBJECT(resource), storage_domain_elements);
     if (!parsed_ok) {
         return FALSE;
     }
@@ -164,8 +193,6 @@ static void ovirt_storage_domain_class_init(OvirtStorageDomainClass *klass)
     OvirtResourceClass *resource_class = OVIRT_RESOURCE_CLASS(klass);
     GParamSpec *param_spec;
 
-    g_type_class_add_private(klass, sizeof(OvirtStorageDomainPrivate));
-
     resource_class->init_from_xml = ovirt_storage_domain_init_from_xml;
     object_class->dispose = ovirt_storage_domain_dispose;
     object_class->get_property = ovirt_storage_domain_get_property;
@@ -249,105 +276,37 @@ static void ovirt_storage_domain_class_init(OvirtStorageDomainClass *klass)
     g_object_class_install_property(object_class,
                                     PROP_STATE,
                                     param_spec);
+
+    param_spec = g_param_spec_boxed("data-center-ids",
+                                    "Data Center Ids",
+                                    "Ids of Data Centers for this Storage Domain",
+                                    G_TYPE_STRV,
+                                    G_PARAM_READWRITE |
+                                    G_PARAM_STATIC_STRINGS);
+    g_object_class_install_property(object_class,
+                                    PROP_DATA_CENTER_IDS,
+                                    param_spec);
 }
 
 static void ovirt_storage_domain_init(OvirtStorageDomain *domain)
 {
-    domain->priv = OVIRT_STORAGE_DOMAIN_GET_PRIVATE(domain);
+    domain->priv = ovirt_storage_domain_get_instance_private(domain);
 }
 
 G_GNUC_INTERNAL
 OvirtStorageDomain *ovirt_storage_domain_new_from_xml(RestXmlNode *node,
                                                       GError **error)
 {
-    GObject *domain;
-
-    domain = g_initable_new(OVIRT_TYPE_STORAGE_DOMAIN, NULL, error,
-                            "xml-node", node, NULL);
-
+    OvirtResource *domain = ovirt_resource_new_from_xml(OVIRT_TYPE_STORAGE_DOMAIN, node, error);
     return OVIRT_STORAGE_DOMAIN(domain);
 }
 
 OvirtStorageDomain *ovirt_storage_domain_new(void)
 {
-    GObject *domain;
-
-    domain = g_initable_new(OVIRT_TYPE_STORAGE_DOMAIN, NULL, NULL, NULL);
-
+    OvirtResource *domain = ovirt_resource_new(OVIRT_TYPE_STORAGE_DOMAIN);
     return OVIRT_STORAGE_DOMAIN(domain);
 }
 
-typedef struct {
-    const char *xml_node;
-    GType type;
-    const char *prop_name;
-} OvirtXmlElement;
-
-static gboolean
-ovirt_resource_parse_xml(OvirtResource *resource,
-                         RestXmlNode *node,
-                         OvirtXmlElement *elements)
-{
-    g_return_val_if_fail(OVIRT_IS_RESOURCE(resource), FALSE);
-    g_return_val_if_fail(elements != NULL, FALSE);
-
-    for (;elements->xml_node != NULL; elements++) {
-        const char *value_str;
-        GValue value = { 0, };
-
-        value_str = ovirt_rest_xml_node_get_content_from_path(node, elements->xml_node);
-        if (value_str == NULL) {
-            continue;
-        }
-        g_value_init(&value, elements->type);
-
-        if (G_TYPE_IS_ENUM(elements->type)) {
-            int enum_value;
-            enum_value = ovirt_utils_genum_get_value(elements->type,
-                                                     value_str, 0);
-            g_value_set_enum(&value, enum_value);
-        } else if (elements->type == G_TYPE_BOOLEAN) {
-            gboolean bool_value;
-
-            bool_value = ovirt_utils_boolean_from_string(value_str);
-            g_value_set_boolean(&value, bool_value);
-        } else if (elements->type == G_TYPE_UINT64) {
-            guint64 int64_value;
-
-            int64_value = g_ascii_strtoull(value_str, NULL, 0);
-            g_value_set_uint64(&value, int64_value);
-        } else if (g_type_is_a(elements->type, OVIRT_TYPE_RESOURCE)) {
-            GObject *resource_value;
-
-            resource_value = g_initable_new(elements->type, NULL, NULL,
-                                            "xml-node", node, NULL);
-            g_value_set_object(&value, resource_value);
-        }
-        g_object_set_property(G_OBJECT(resource), elements->prop_name, &value);
-        g_value_unset(&value);
-    }
-
-    return TRUE;
-}
-
-static gboolean
-ovirt_storage_domain_refresh_from_xml(OvirtStorageDomain *domain,
-                                      RestXmlNode *node)
-{
-    OvirtXmlElement storage_domain_elements[] = {
-        { "type",           OVIRT_TYPE_STORAGE_DOMAIN_TYPE,           "type" },
-        { "master",         G_TYPE_BOOLEAN,                           "master" },
-        { "available",      G_TYPE_UINT64,                            "available" },
-        { "used",           G_TYPE_UINT64,                            "used" },
-        { "committed",      G_TYPE_UINT64,                            "committed" },
-        { "storage_format", OVIRT_TYPE_STORAGE_DOMAIN_FORMAT_VERSION, "version" },
-        { "status/state",   OVIRT_TYPE_STORAGE_DOMAIN_STATE,          "state" },
-        { NULL,             G_TYPE_INVALID,                           NULL }
-    };
-
-    return ovirt_resource_parse_xml(OVIRT_RESOURCE(domain), node, storage_domain_elements);
-}
-
 
 /**
  * ovirt_storage_domain_get_files:
@@ -363,20 +322,14 @@ ovirt_storage_domain_refresh_from_xml(OvirtStorageDomain *domain,
  */
 OvirtCollection *ovirt_storage_domain_get_files(OvirtStorageDomain *domain)
 {
-    const char *href;
-
     g_return_val_if_fail(OVIRT_IS_STORAGE_DOMAIN(domain), NULL);
 
-    if (domain->priv->files != NULL)
-        return domain->priv->files;
-
-    href = ovirt_resource_get_sub_collection(OVIRT_RESOURCE(domain), "files");
-    if (href == NULL)
-        return NULL;
-
-    domain->priv->files = ovirt_collection_new(href, "files",
-                                               OVIRT_TYPE_RESOURCE,
-                                               "file");
+    if (domain->priv->files == NULL)
+        domain->priv->files = ovirt_sub_collection_new_from_resource(OVIRT_RESOURCE(domain),
+                                                                     "files",
+                                                                     "files",
+                                                                     OVIRT_TYPE_RESOURCE,
+                                                                     "file");
 
     return domain->priv->files;
 }
