@@ -24,9 +24,7 @@
 
 #include "ovirt-enum-types.h"
 #include "ovirt-vm-display.h"
-
-#define OVIRT_VM_DISPLAY_GET_PRIVATE(obj)                         \
-        (G_TYPE_INSTANCE_GET_PRIVATE((obj), OVIRT_TYPE_VM_DISPLAY, OvirtVmDisplayPrivate))
+#include "ovirt-utils.h"
 
 struct _OvirtVmDisplayPrivate {
     OvirtVmDisplayType type;
@@ -43,7 +41,7 @@ struct _OvirtVmDisplayPrivate {
     char *proxy_url;
 };
 
-G_DEFINE_TYPE(OvirtVmDisplay, ovirt_vm_display, G_TYPE_OBJECT);
+G_DEFINE_TYPE_WITH_PRIVATE(OvirtVmDisplay, ovirt_vm_display, G_TYPE_OBJECT);
 
 enum {
     PROP_0,
@@ -185,8 +183,6 @@ static void ovirt_vm_display_class_init(OvirtVmDisplayClass *klass)
 {
     GObjectClass *object_class = G_OBJECT_CLASS(klass);
 
-    g_type_class_add_private(klass, sizeof(OvirtVmDisplayPrivate));
-
     object_class->finalize = ovirt_vm_display_finalize;
     object_class->get_property = ovirt_vm_display_get_property;
     object_class->set_property = ovirt_vm_display_set_property;
@@ -197,7 +193,7 @@ static void ovirt_vm_display_class_init(OvirtVmDisplayClass *klass)
                                                         "Type",
                                                         "Display Type",
                                                         OVIRT_TYPE_VM_DISPLAY_TYPE,
-                                                        OVIRT_VM_DISPLAY_SPICE,
+                                                        OVIRT_VM_DISPLAY_INVALID,
                                                         G_PARAM_READWRITE |
                                                         G_PARAM_STATIC_STRINGS));
     g_object_class_install_property(object_class,
@@ -296,10 +292,69 @@ static void ovirt_vm_display_class_init(OvirtVmDisplayClass *klass)
 
 static void ovirt_vm_display_init(G_GNUC_UNUSED OvirtVmDisplay *display)
 {
-    display->priv = OVIRT_VM_DISPLAY_GET_PRIVATE(display);
+    display->priv = ovirt_vm_display_get_instance_private(display);
 }
 
 OvirtVmDisplay *ovirt_vm_display_new(void)
 {
     return OVIRT_VM_DISPLAY(g_object_new(OVIRT_TYPE_VM_DISPLAY, NULL));
 }
+
+static gboolean ovirt_vm_display_set_from_xml(OvirtVmDisplay *display, RestXmlNode *node)
+{
+    OvirtVmDisplayType type;
+    OvirtXmlElement display_elements[] = {
+        { .prop_name = "type",
+          .xml_path = "type",
+        },
+        { .prop_name = "address",
+          .xml_path = "address",
+        },
+        { .prop_name = "port",
+          .xml_path = "port",
+        },
+        { .prop_name = "secure-port",
+          .xml_path = "secure_port",
+        },
+        { .prop_name = "monitor-count",
+          .xml_path = "monitors",
+        },
+        { .prop_name = "smartcard",
+          .xml_path = "smartcard_enabled",
+        },
+        { .prop_name = "allow-override",
+          .xml_path = "allow_override",
+        },
+        { .prop_name = "host-subject",
+          .xml_path = "certificate/subject",
+        },
+        { .prop_name = "proxy-url",
+          .xml_path = "proxy",
+        },
+        { NULL, },
+    };
+
+    ovirt_rest_xml_node_parse(node, G_OBJECT(display), display_elements);
+    g_object_get(G_OBJECT(display), "type", &type, NULL);
+    if (type == OVIRT_VM_DISPLAY_INVALID) {
+        return FALSE;
+    }
+
+    return TRUE;
+}
+
+OvirtVmDisplay *ovirt_vm_display_new_from_xml(RestXmlNode *node)
+{
+    OvirtVmDisplay *display;
+
+    g_return_val_if_fail(node != NULL, NULL);
+
+    display = ovirt_vm_display_new();
+
+    if (!ovirt_vm_display_set_from_xml(display, node)) {
+        g_object_unref(display);
+        return NULL;
+    }
+
+    return display;
+}
